==> warcar-1.0.0-20171226.024301-23-sources.jar: com/bxm/warcar/cache/impls/redis/JedisFetcher.java <==
package com.bxm.warcar.cache.impls.redis;

import com.bxm.warcar.cache.*;
import com.bxm.warcar.cache.serialization.JSONSerialization;
import com.google.common.collect.Maps;
import org.apache.commons.lang.StringUtils;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;

import java.util.List;
import java.util.Map;
import java.util.Set;

public class JedisFetcher implements Fetcher {

    public static final String HASH_ALL_FIELD = "____ALL____";

    private final JedisPool jedisPool;
    private final JSONSerialization serialization;
    /**
     * 是否启用ALL_FIELD实现
     */
    private boolean enableAllField = true;
    private String allFieldName = HASH_ALL_FIELD;

    public JedisFetcher(JedisPool jedisPool) {
        this.jedisPool = jedisPool;
        this.serialization = new JSONSerialization();
    }

    @Override
    public <T> T fetch(KeyGenerator keyGenerator, DataExtractor<T> dataExtractor, Class<T> cls) {
        return fetch(keyGenerator, dataExtractor, cls, 0);
    }

    @Override
    public <T> T fetch(KeyGenerator keyGenerator, DataExtractor<T> dataExtractor, Class<T> cls, int expireTimeInSecond) {
        if (null == keyGenerator) {
            throw new NullPointerException("getKeyGenerator");
        }

        String key = keyGenerator.generateKey();
        if (StringUtils.isBlank(key)) {
            throw new NullPointerException("key");
        }

        Jedis jedis = null;
        try {
            jedis = jedisPool.getResource();

            String value = jedis.get(key);

            if (null != value) {
                return serialization.deserialize(value, cls);
            }

            T fromStore = null;
            if (null != dataExtractor) {
                fromStore = dataExtractor.extract();
            }

            if (null != fromStore) {
                jedis.set(key, serialization.serialize(fromStore));
                if (expireTimeInSecond > 0)
                    jedis.expire(key, expireTimeInSecond);
            }

            return fromStore;
        }
        finally {
            if (null != jedis) {
                jedis.close();
            }
        }
    }

    @Override
    public <T> List<T> fetchList(KeyGenerator keyGenerator, DataExtractor<List<T>> dataExtractor, Class<T> cls) {
        return fetchList(keyGenerator, dataExtractor, cls, 0);
    }

    @Override
    public <T> List<T> fetchList(KeyGenerator keyGenerator, DataExtractor<List<T>> dataExtractor, Class<T> cls, int expireTimeInSecond) {
        if (null == keyGenerator) {
            throw new NullPointerException("getKeyGenerator");
        }

        String key = keyGenerator.generateKey();
        if (StringUtils.isBlank(key)) {
            throw new NullPointerException("key");
        }

        Jedis jedis = null;
        try {
            jedis = jedisPool.getResource();

            String value = jedis.get(key);

            if (null != value) {
                return serialization.deserializeList(value, cls);
            }

            List<T> fromStore = null;
            if (null != dataExtractor) {
                fromStore = dataExtractor.extract();
            }

            if (null != fromStore) {
                jedis.set(key, serialization.serialize(fromStore));
                if (expireTimeInSecond > 0)
                    jedis.expire(key, expireTimeInSecond);
            }

            return fromStore;
        }
        finally {
            if (null != jedis) {
                jedis.close();
            }
        }
    }

    @Override
    public <T> T hfetch(KeyGenerator keyGenerator, String field, DataExtractor<T> dataExtractor, Class<T> cls) {
        return this.hfetch(keyGenerator, field, dataExtractor, cls, 0);
    }

    @Override
    public <T> T hfetch(KeyGenerator keyGenerator, String field, DataExtractor<T> dataExtractor, Class<T> cls, int expireTimeInSecond) {
        if (null == keyGenerator) {
            throw new NullPointerException("getKeyGenerator");
        }

        String key = keyGenerator.generateKey();
        if (StringUtils.isBlank(key)) {
            throw new NullPointerException("key");
        }
        if (StringUtils.isBlank(field)) {
            throw new NullPointerException("field");
        }

        Jedis jedis = null;
        try {
            jedis = jedisPool.getResource();

            String value = jedis.hget(key, field);

            if (null != value) {
                return serialization.deserialize(value, cls);
            }

            T fromStore = null;
            if (null != dataExtractor) {
                fromStore = dataExtractor.extract();
            }

            if (null != fromStore) {
                jedis.hset(key, field, serialization.serialize(fromStore));
                if (expireTimeInSecond > 0)
                    jedis.expire(key, expireTimeInSecond);
            }

            return fromStore;
        }
        finally {
            if (null != jedis) {
                jedis.close();
            }
        }
    }

    @Override
    public <T> Map<String, T> hfetchall(KeyGenerator keyGenerator, DataExtractor<Map<String, T>> dataExtractor, Class<T> cls) {
        return hfetchall(keyGenerator, dataExtractor, cls, 0);
    }

    /**
     * Jedis 的hgetall实现，为了解决hgetall命令的性能问题。在该命令上加了一层缓存。
     * <pre>
     *     String allValue = jedis.hget(key, "ALL_FIELD_NAME");
     *     if (null == allValue) {
     *         jedis.hgetall(key);
     *     }
     * </pre>
     * 这个功能需要{@link JedisUpdater}配合使用，当更新这个hash中的任意一个field值时，需要刷新这个<code>ALL_FIELD_NAME</code>对应的值，以保证数据的一致性。
     * <b>更新频次高的谨慎使用。</b>
     *
     * @param keyGenerator
     * @param dataExtractor
     * @param cls
     * @param expireTimeInSecond
     * @param <T>
     * @return
     */
    @Override
    public <T> Map<String, T> hfetchall(KeyGenerator keyGenerator, DataExtractor<Map<String, T>> dataExtractor, Class<T> cls, int expireTimeInSecond) {
        if (null == keyGenerator) {
            throw new NullPointerException("getKeyGenerator");
        }

        String key = keyGenerator.generateKey();
        if (StringUtils.isBlank(key)) {
            throw new NullPointerException("key");
        }

        Jedis jedis = null;
        try {
            jedis = jedisPool.getResource();

            Map<String, T> rst = Maps.newHashMap();

            Map<String, String> value = null;

            if (enableAllField) {
                String allValFromCache = jedis.hget(key, allFieldName);
                if (StringUtils.isNotBlank(allValFromCache)) {
                    return serialization.deserializeMap(allValFromCache, cls);
                }
            }

            value = jedis.hgetAll(key);

            if (null != value) {
                Set<Map.Entry<String, String>> entries = value.entrySet();
                for (Map.Entry<String, String> entry : entries) {
                    String json = entry.getValue();
                    T deserialize = serialization.deserialize(json, cls);
                    rst.put(entry.getKey(), deserialize);
                }

                // save to ALL
                jedis.hset(key, allFieldName, serialization.serialize(rst));

                return rst;
            }

            Map<String, T> fromStore = null;
            if (null != dataExtractor) {
                fromStore = dataExtractor.extract();
            }

            if (null != fromStore) {
                Set<Map.Entry<String, T>> entries = fromStore.entrySet();
                for (Map.Entry<String, T> entry : entries) {
                    jedis.hset(key, entry.getKey(), serialization.serialize(entry.getValue()));
                }
                if (expireTimeInSecond > 0)
                    jedis.expire(key, expireTimeInSecond);
            }

            return fromStore;
        }
        finally {
            if (null != jedis) {
                jedis.close();
            }
        }
    }

    public boolean isEnableAllField() {
        return enableAllField;
    }

    public void setEnableAllField(boolean enableAllField) {
        this.enableAllField = enableAllField;
    }

    public String getAllFieldName() {
        return allFieldName;
    }

    public void setAllFieldName(String allFieldName) {
        this.allFieldName = allFieldName;
    }
}

==> warcar-1.0.2-20180312.140057-1-sources.jar: com/bxm/warcar/cache/impls/redis/JedisFetcher.java <==
package com.bxm.warcar.cache.impls.redis;

import com.bxm.warcar.cache.DataExtractor;
import com.bxm.warcar.cache.Fetcher;
import com.bxm.warcar.cache.KeyGenerator;
import com.bxm.warcar.cache.serialization.JSONSerialization;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.ScanParams;
import redis.clients.jedis.ScanResult;

import java.util.List;
import java.util.Map;
import java.util.Set;

public class JedisFetcher implements Fetcher {

    private final JedisPool jedisPool;
    private final JSONSerialization serialization;

    public JedisFetcher(JedisPool jedisPool) {
        this.jedisPool = jedisPool;
        this.serialization = new JSONSerialization();
    }

    @Override
    public Object getClientOriginal() {
        return jedisPool;
    }

    @Override
    public <T> T fetch(KeyGenerator keyGenerator, Class<T> cls) {
        return fetch(keyGenerator, null, cls);
    }

    @Override
    public <T> T fetch(KeyGenerator keyGenerator, Class<T> cls, int expireTimeInSecond) {
        return fetch(keyGenerator, null, cls, expireTimeInSecond);
    }

    @Override
    public <T> T fetch(KeyGenerator keyGenerator, DataExtractor<T> dataExtractor, Class<T> cls) {
        return fetch(keyGenerator, dataExtractor, cls, 0);
    }

    @Override
    public <T> T fetch(KeyGenerator keyGenerator, DataExtractor<T> dataExtractor, Class<T> cls, int expireTimeInSecond) {
        if (null == keyGenerator) {
            throw new NullPointerException("getKeyGenerator");
        }

        String key = keyGenerator.generateKey();
        if (StringUtils.isBlank(key)) {
            throw new NullPointerException("key");
        }

        Jedis jedis = null;
        try {
            jedis = jedisPool.getResource();

            String value = jedis.get(key);

            if (null != value) {
                return serialization.deserialize(value, cls);
            }

            T fromStore = null;
            if (null != dataExtractor) {
                fromStore = dataExtractor.extract();
            }

            if (null != fromStore) {
                jedis.set(key, serialization.serialize(fromStore));
                if (expireTimeInSecond > 0)
                    jedis.expire(key, expireTimeInSecond);
            }

            return fromStore;
        }
        finally {
            if (null != jedis) {
                jedis.close();
            }
        }
    }

    @Override
    public <T> List<T> fetchList(KeyGenerator keyGenerator, Class<T> cls) {
        return fetchList(keyGenerator, null, cls);
    }

    @Override
    public <T> List<T> fetchList(KeyGenerator keyGenerator, DataExtractor<List<T>> dataExtractor, Class<T> cls) {
        return fetchList(keyGenerator, dataExtractor, cls, 0);
    }

    @Override
    public <T> List<T> fetchList(KeyGenerator keyGenerator, Class<T> cls, int expireTimeInSecond) {
        return fetchList(keyGenerator, null, cls, expireTimeInSecond);
    }

    @Override
    public <T> List<T> fetchList(KeyGenerator keyGenerator, DataExtractor<List<T>> dataExtractor, Class<T> cls, int expireTimeInSecond) {
        if (null == keyGenerator) {
            throw new NullPointerException("getKeyGenerator");
        }

        String key = keyGenerator.generateKey();
        if (StringUtils.isBlank(key)) {
            throw new NullPointerException("key");
        }

        Jedis jedis = null;
        try {
            jedis = jedisPool.getResource();

            String value = jedis.get(key);

            if (null != value) {
                return serialization.deserializeList(value, cls);
            }

            List<T> fromStore = null;
            if (null != dataExtractor) {
                fromStore = dataExtractor.extract();
            }

            if (null != fromStore) {
                jedis.set(key, serialization.serialize(fromStore));
                if (expireTimeInSecond > 0)
                    jedis.expire(key, expireTimeInSecond);
            }

            return fromStore;
        }
        finally {
            if (null != jedis) {
                jedis.close();
            }
        }
    }

    @Override
    public <T> T hfetch(KeyGenerator keyGenerator, String field, Class<T> cls) {
        return hfetch(keyGenerator, field, null, cls);
    }

    @Override
    public <T> T hfetch(KeyGenerator keyGenerator, String field, Class<T> cls, int expireTimeInSecond) {
        return hfetch(keyGenerator, field, null, cls, expireTimeInSecond);
    }

    @Override
    public <T> T hfetch(KeyGenerator keyGenerator, String field, DataExtractor<T> dataExtractor, Class<T> cls) {
        return this.hfetch(keyGenerator, field, dataExtractor, cls, 0);
    }

    @Override
    public <T> T hfetch(KeyGenerator keyGenerator, String field, DataExtractor<T> dataExtractor, Class<T> cls, int expireTimeInSecond) {
        if (null == keyGenerator) {
            throw new NullPointerException("getKeyGenerator");
        }

        String key = keyGenerator.generateKey();
        if (StringUtils.isBlank(key)) {
            throw new NullPointerException("key");
        }
        if (StringUtils.isBlank(field)) {
            throw new NullPointerException("field");
        }

        Jedis jedis = null;
        try {
            jedis = jedisPool.getResource();

            String value = jedis.hget(key, field);

            if (null != value) {
                return serialization.deserialize(value, cls);
            }

            T fromStore = null;
            if (null != dataExtractor) {
                fromStore = dataExtractor.extract();
            }

            if (null != fromStore) {
                jedis.hset(key, field, serialization.serialize(fromStore));
                if (expireTimeInSecond > 0)
                    jedis.expire(key, expireTimeInSecond);
            }

            return fromStore;
        }
        finally {
            if (null != jedis) {
                jedis.close();
            }
        }
    }

    @Override
    public <T> List<T> hfetchList(KeyGenerator keyGenerator, String field, Class<T> cls) {
        return hfetchList(keyGenerator, field, null, cls);
    }

    @Override
    public <T> List<T> hfetchList(KeyGenerator keyGenerator, String field, Class<T> cls, int expireTimeInSecond) {
        return hfetchList(keyGenerator, field, null, cls, expireTimeInSecond);
    }

    @Override
    public <T> List<T> hfetchList(KeyGenerator keyGenerator, String field, DataExtractor<List<T>> dataExtractor, Class<T> cls) {
        return hfetchList(keyGenerator, field, dataExtractor, cls, 0);
    }

    @Override
    public <T> List<T> hfetchList(KeyGenerator keyGenerator, String field, DataExtractor<List<T>> dataExtractor, Class<T> cls, int expireTimeInSecond) {
        if (null == keyGenerator) {
            throw new NullPointerException("getKeyGenerator");
        }

        String key = keyGenerator.generateKey();
        if (StringUtils.isBlank(key)) {
            throw new NullPointerException("key");
        }
        if (StringUtils.isBlank(field)) {
            throw new NullPointerException("field");
        }

        Jedis jedis = null;
        try {
            jedis = jedisPool.getResource();

            String value = jedis.hget(key, field);

            if (null != value) {
                return serialization.deserializeList(value, cls);
            }

            List<T> fromStore = null;
            if (null != dataExtractor) {
                fromStore = dataExtractor.extract();
            }

            if (null != fromStore) {
                jedis.hset(key, field, serialization.serialize(fromStore));
                if (expireTimeInSecond > 0)
                    jedis.expire(key, expireTimeInSecond);
            }

            return fromStore;
        }
        finally {
            if (null != jedis) {
                jedis.close();
            }
        }
    }


    @Override
    public <T> Map<String, T> hfetchall(KeyGenerator keyGenerator, Class<T> cls) {
        return hfetchall(keyGenerator, null, cls);
    }

    @Override
    public <T> Map<String, T> hfetchall(KeyGenerator keyGenerator, Class<T> cls, int expireTimeInSecond) {
        return hfetchall(keyGenerator, null, cls, expireTimeInSecond);
    }

    @Override
    public <T> Map<String, T> hfetchall(KeyGenerator keyGenerator, DataExtractor<Map<String, T>> dataExtractor, Class<T> cls) {
        return hfetchall(keyGenerator, dataExtractor, cls, 0);
    }

    /**
     * <p>
     *  Jedis 的hgetall实现，为了解决hgetall命令的性能问题。内部使用增量式迭代命令<code>hscan</code>来实现。
     * </p>
     * <b>增量式迭代命令只能对被返回的元素提供有限的保证 （offer limited guarantees about the returned elements）</b>
     * <p>
     *     举个例子， 使用 SMEMBERS 命令可以返回集合键当前包含的所有元素， 但是对于 SCAN 这类增量式迭代命令来说， 因为在对键进行增量式迭代的过程中， 键可能会被修改。
     * </p>
     * <p>
     *     因为增量式命令仅仅使用游标来记录迭代状态， 所以这些命令带有以下缺点：
     * </p>
     * <ul>
     * <li>
     *     同一个元素可能会被返回多次。 处理重复元素的工作交由应用程序负责， 比如说， 可以考虑将迭代返回的元素仅仅用于可以安全地重复执行多次的操作上。
     * </li>
     * <li>
     *     如果一个元素是在迭代过程中被添加到数据集的， 又或者是在迭代过程中从数据集中被删除的， 那么这个元素可能会被返回， 也可能不会， 这是未定义的（undefined）。
     * </li>
     * </ul>
     * <p>
     *     更多说明请参考：<a href="http://doc.redisfans.com/key/scan.html">http://doc.redisfans.com/key/scan.html</a>
     * </p>
     *
     * @param keyGenerator
     * @param dataExtractor
     * @param cls
     * @param expireTimeInSecond
     * @param <T>
     * @return
     */
    @Override
    public <T> Map<String, T> hfetchall(KeyGenerator keyGenerator, DataExtractor<Map<String, T>> dataExtractor, Class<T> cls, int expireTimeInSecond) {
        if (null == keyGenerator) {
            throw new NullPointerException("getKeyGenerator");
        }

        String key = keyGenerator.generateKey();
        if (StringUtils.isBlank(key)) {
            throw new NullPointerException("key");
        }

        Jedis jedis = null;
        try {
            jedis = jedisPool.getResource();

            Map<String, T> rst = Maps.newHashMap();
            final String startCursor = "0";
            String cursor = startCursor;
            do {
                ScanResult<Map.Entry<String, String>> hscan = jedis.hscan(key, cursor, new ScanParams().count(10000));
                cursor = hscan.getStringCursor();
                List<Map.Entry<String, String>> result = hscan.getResult();
                if (CollectionUtils.isNotEmpty(result)) {
                    for (Map.Entry<String, String> entry : result) {
                        String k = entry.getKey();
                        String v = entry.getValue();
                        rst.put(k, serialization.deserialize(v, cls));
                    }
                }
            } while (! cursor.equals(startCursor));

            if (MapUtils.isNotEmpty(rst))
                return rst;

            Map<String, T> fromStore = null;
            if (null != dataExtractor) {
                fromStore = dataExtractor.extract();
            }

            if (null != fromStore) {
                Set<Map.Entry<String, T>> entries = fromStore.entrySet();
                for (Map.Entry<String, T> entry : entries) {
                    jedis.hset(key, entry.getKey(), serialization.serialize(entry.getValue()));
                }
                if (expireTimeInSecond > 0)
                    jedis.expire(key, expireTimeInSecond);
            }

            return fromStore;
        }
        finally {
            if (null != jedis) {
                jedis.close();
            }
        }
    }
}
